//
//  CalendarRepository.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 2/5/21.
//

import UIKit
import Foundation
import CoreData

class CalendarRepository {
  static let shared = CalendarRepository()

  init() {}

  init(withContext: NSManagedObjectContext) {
    context = withContext
  }

  // swiftlint:disable:next force_cast
  private var context = (UIApplication.shared.delegate as! AppDelegate).persistentContainer.viewContext

  func fetch(in date: Date, completion: @escaping (CalendarDay?) -> Void) {
    let fetchRequest = NSFetchRequest<NSFetchRequestResult>(entityName: "CalendarDay")
    // Get the current calendar with local time zone
    var calendar = Calendar.current
    calendar.timeZone = NSTimeZone.local

    // Get today's beginning & end
    let dateFrom = calendar.startOfDay(for: date)
    let dateTo = calendar.date(byAdding: .day, value: 1, to: dateFrom)

    // swiftlint:disable:next force_unwrapping
    let predicate = NSPredicate(format: "date >= %@ && date <= %@", dateFrom as NSDate, dateTo! as NSDate)

    fetchRequest.predicate = predicate
    fetchRequest.fetchLimit = 1

    do {
      let fetchedResult = try context.fetch(fetchRequest)
      if let result = fetchedResult as? [CalendarDay],
        let day = result.first {
        completion(day)
        return
      }
    } catch {
      assertionFailure("Error recogiendo el día del calendario \(error.localizedDescription)")
    }

    completion(nil)
  }

  func fetch(in dates: [Date], completion: @escaping ([CalendarDay]?) -> Void) {
    guard !dates.isEmpty else { return }
    let fetchRequest = NSFetchRequest<NSFetchRequestResult>(entityName: "CalendarDay")
    // Get the current calendar with local time zone
    var calendar = Calendar.current
    calendar.timeZone = NSTimeZone.local
    // swiftlint:disable force_unwrapping
    // Get today's beginning & end
    let sortedDates = dates.sorted()
    let dateFrom = calendar.startOfDay(for: sortedDates.first!)
    let dateTo = calendar.date(byAdding: .day, value: 1, to: sortedDates.last!)!

    let predicate = NSPredicate(format: "date >= %@ && date <= %@", dateFrom as NSDate, dateTo as NSDate)
    // swiftlint:enable force_unwrapping
    fetchRequest.predicate = predicate

    do {
      let fetchedResult = try context.fetch(fetchRequest)
      if let result = fetchedResult as? [CalendarDay] {
        completion(result)
        return
      }
    } catch {
      assertionFailure("Error recogiendo el día del calendario \(error.localizedDescription)")
    }

    completion(nil)
  }

  func createNewDay(for date: Date) -> CalendarDay {
    let newCalendarDay = CalendarDay(context: context)
    newCalendarDay.id = UUID()
    newCalendarDay.date = date
    return newCalendarDay
  }

  func createNewCalendarItem() -> CalendarItem {
    let newItem = CalendarItem(context: context)
    newItem.id = UUID()
    return newItem
  }

  func save() {
    saveContext { _ in
    }
  }

  func delete(element: CalendarItem, in day: CalendarDay) {
    day.removeFromItems(element)

    do {
      try context.save()
    } catch {}
  }

  private func saveContext(completion: @escaping (Bool) -> Void) {
    do {
      try context.save()
      completion(true)
    } catch {
      assertionFailure("Error guardando el día \(error.localizedDescription)")
      completion(false)
    }
  }
}
