//
//  DishesRepository.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 1/5/21.
//

import UIKit
import CoreData

class DishesRepository {
  static let shared = DishesRepository()

  init() {}

  init(withContext: NSManagedObjectContext) {
    context = withContext
  }

  // swiftlint:disable:next force_cast
  private var context = (UIApplication.shared.delegate as! AppDelegate).persistentContainer.viewContext

  func fetch(completion: @escaping ([Dish]) -> Void) {
    do {
      let dishes: [Dish] = try context.fetch(Dish.fetchRequest())
      completion(dishes)
    } catch {
      assertionFailure("Error leyendo los artículos")
      completion([])
    }
  }

  func save(
    withName name: String,
    color: UIColor,
    andArticles articles: [Article],
    completion: @escaping (Bool) -> Void
  ) {
    let newDish = Dish(context: context)
    newDish.id = UUID()
    newDish.color = color as NSObject
    newDish.name = name

    articles.forEach { newDish.addToArticles($0) }

    saveContext(completion: completion)
  }

  func createDish() -> Dish {
    let newDish = Dish(context: context)
    newDish.id = UUID()
    return newDish
  }

  func delete(dish: Dish) {
    context.delete(dish)

    do {
      try context.save()
    } catch {}
  }

  func updateDishes(completion: @escaping (Bool) -> Void) {
    saveContext(completion: completion)
  }

  func discard(dish: Dish) {
    context.refresh(dish, mergeChanges: false)
  }

  private func saveContext(completion: @escaping (Bool) -> Void) {
    do {
      try context.save()
      completion(true)
    } catch {
      assertionFailure("Error guardando el artículo \(error.localizedDescription)")
      completion(false)
    }
  }
}
