//
//  ShoppingListRepository.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 3/5/21.
//

import UIKit
import CoreData

class ShoppingListRepository {
  static let shared = ShoppingListRepository()

  init() {}

  init(withContext: NSManagedObjectContext) {
    context = withContext
  }

  // swiftlint:disable:next force_cast
  private var context = (UIApplication.shared.delegate as! AppDelegate).persistentContainer.viewContext

  func createManualList() -> ShoppingList {
    // manual list creation
    let newList = ShoppingList(context: context)
    newList.id = UUID()
    newList.isPurchased = false
    newList.listType = ListsViewModel.ShoppingListType.manual.rawValue
    newList.dates = [Date()]
    return newList
  }

  func createAutomaticList(forDate date: Date) -> ShoppingList {
    let newList = ShoppingList(context: context)
    newList.id = UUID()
    newList.name = NSLocalizedString("Weekly list", comment: "")
    newList.isPurchased = false
    newList.listType = ListsViewModel.ShoppingListType.automatic.rawValue
    newList.dates = date.getDaysOfWeek()
    return newList
  }

  func createItem() -> ShoppingListItem {
    let newItem = ShoppingListItem(context: context)
    newItem.id = UUID()
    newItem.isPurchased = false
    newItem.didDiscardItem = false
    return newItem
  }

  func fetch(completion: @escaping ([ShoppingList]) -> Void) {
    do {
      let lists: [ShoppingList] = try context.fetch(ShoppingList.fetchRequest())
      completion(lists)
    } catch {
      assertionFailure("Error leyendo las listas de la compra")
      completion([])
    }
  }

  func fetchAutomaticLists(completion: @escaping ([ShoppingList]) -> Void) {
    do {
      let fetchRequest = NSFetchRequest<NSFetchRequestResult>(entityName: "ShoppingList")
      // swiftlint:disable:next compiler_protocol_init
      let number = NSNumber(integerLiteral: Int(ListsViewModel.ShoppingListType.automatic.rawValue))
      let predicate = NSPredicate(
        format: "listType = %@", number)
      fetchRequest.predicate = predicate

      let fetchedResult = try context.fetch(fetchRequest)
      if let result = fetchedResult as? [ShoppingList] {
        completion(result)
      }
    } catch {
      assertionFailure("Error leyendo las listas de la compra")
      completion([])
    }
  }

  func save(completion: @escaping (Bool) -> Void) {
    saveContext(completion: completion)
  }

  func delete(list: ShoppingList) {
    context.delete(list)

    do {
      try context.save()
    } catch {}
  }

  func discard(list: ShoppingList) {
    context.refresh(list, mergeChanges: false)
  }

  private func saveContext(completion: @escaping (Bool) -> Void) {
    do {
      try context.save()
      completion(true)
    } catch {
      assertionFailure("Error guardando el artículo \(error.localizedDescription)")
      completion(false)
    }
  }
}
