//
//  Date+Extension.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 17/4/21.
//

import Foundation

extension Date {
  var dayNumber: Int {
    let components = Calendar.current.dateComponents([.day], from: self)
    return components.day ?? 1
  }

  var dayOfWeekAbbr: String {
    let calendar = Calendar.current
    let index = calendar.component(.weekday, from: self) - 1
    let dayName = calendar.shortWeekdaySymbols[index].capitalized
    return dayName
  }

  var monthNumber: Int {
    let components = Calendar.current.dateComponents([.month], from: self)
    return components.month ?? 1
  }

  var yearNumber: Int {
    let components = Calendar.current.dateComponents([.year], from: self)
    return components.year ?? 1
  }

  func daysInMonth() -> Int {
    var calendar = Calendar.current
    // swiftlint:disable:next force_unwrapping
    calendar.timeZone = TimeZone(identifier: "Europe/Madrid")!
    let dateComponents = DateComponents(
      year: calendar.component(.year, from: self),
      month: calendar.component(.month, from: self))

    if let date = calendar.date(from: dateComponents), let range = calendar.range(of: .day, in: .month, for: date) {
      return range.count
    }
    return 0
  }

  func getDaysOfMonth() -> [Date] {
    var calendar = Calendar.current
    // swiftlint:disable:next force_unwrapping
    calendar.timeZone = TimeZone(identifier: "Europe/Madrid")!

    var dateComponents = DateComponents()
    dateComponents.day = 1
    dateComponents.month = monthNumber
    dateComponents.year = yearNumber
    dateComponents.hour = 6
    dateComponents.minute = 4
    // swiftlint:disable:next force_unwrapping
    let firstDayDate = calendar.date(from: dateComponents)!
    let monthDays = firstDayDate.daysInMonth()

    var days: [Date] = []

    for nextDay in 0..<monthDays {
      if let newDate = calendar.date(byAdding: .day, value: nextDay, to: firstDayDate) {
        days.append(newDate)
      }
    }

    // complete first and last week if not already completed
    if let firstDayDate = days.first, let lastDayDate = days.last {
      let firstWeek = firstDayDate.getDaysOfWeek()
      let firstAdditionalDays = firstWeek.filter { $0.monthNumber != firstDayDate.monthNumber }

      let lastWeek = lastDayDate.getDaysOfWeek()
      let lastAdittionalDays = lastWeek.filter { $0.monthNumber != firstDayDate.monthNumber }
      days.append(contentsOf: firstAdditionalDays)
      days.append(contentsOf: lastAdittionalDays)
    }

    days = days.sorted()

    return days
  }

  func getDaysOfWeek() -> [Date] {
    var calendar = Calendar.current
    // swiftlint:disable:next force_unwrapping
    calendar.timeZone = TimeZone(identifier: "Europe/Madrid")!
    let today = calendar.startOfDay(for: self)
    let dayOfWeek = calendar.component(.weekday, from: today)

    let weekSum = dayOfWeek == 1 ? -7 : -dayOfWeek + 1

    // swiftlint:disable:next force_unwrapping
    let weekdays = calendar.range(of: .weekday, in: .weekOfYear, for: today)!

    // + 1 is added to go from monday to sunday instead of from sunday to saturday
    let days = (weekdays.lowerBound ..< weekdays.upperBound)
      .compactMap { calendar.date(byAdding: .day, value: $0 + weekSum, to: today) }

    return days
  }
}
