//
//  UIColorValueTransformer.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 28/4/21.
//

import UIKit

// https://www.avanderlee.com/swift/valuetransformer-core-data/

@objc(UIColorValueTransformer)
public final class UIColorValueTransformer: ValueTransformer {
  public override class func transformedValueClass() -> AnyClass {
    UIColor.self
  }

  public override class func allowsReverseTransformation() -> Bool {
    true
  }

  public override func transformedValue(_ value: Any?) -> Any? {
    guard let color = value as? UIColor else { return nil }

    do {
      let data = try NSKeyedArchiver.archivedData(
        withRootObject: color,
        requiringSecureCoding: true)
      return data
    } catch {
      assertionFailure("Error al transformar de UIColor")
      return nil
    }
  }

  public override func reverseTransformedValue(_ value: Any?) -> Any? {
    guard let data = value as? NSData else { return nil }

    do {
      let color = try NSKeyedUnarchiver.unarchivedObject(
        ofClass: UIColor.self,
        from: data as Data)
      return color
    } catch {
      assertionFailure("Error al transformar a UIColor")
      return nil
    }
  }
}

extension UIColorValueTransformer {
  static let name = NSValueTransformerName(rawValue: String(describing: UIColorValueTransformer.self))

  public static func register() {
    let transformer = UIColorValueTransformer()
    ValueTransformer.setValueTransformer(transformer, forName: name)
  }
}
