//
//  ArticlesViewModel.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 8/4/21.
//

import UIKit

class ArticlesViewModel: SimpleListViewModelProtocol {
  private let repository = ArticlesRepository.shared

  var groupHeight: CGFloat = 70
  var navigationItemTitle: String = NSLocalizedString("Articles", comment: "")
  var type: ListType = .articles

  var items: [AnyHashable] = [] {
    didSet {
      itemsDidChange?(items)
    }
  }

  var itemsDidChange: (([AnyHashable]) -> Void)?

  deinit {
    NotificationCenter.default.removeObserver(
      self,
      name: Notification.Name.NSManagedObjectContextObjectsDidChange,
      object: nil)
  }

  init() {
    NotificationCenter.default.addObserver(
      self,
      selector: #selector(modelDidChange),
      name: Notification.Name.NSManagedObjectContextObjectsDidChange,
      object: nil)
  }

  func fetch() {
    repository.fetch { articles in
      self.items = articles
    }
  }

  func delete<T>(_ item: T) {
    if let article = item as? Article {
      repository.delete(article: article)
    }
  }

  @objc private func modelDidChange() {
    repository.fetch { articles in
      // swiftlint:disable force_unwrapping
      let sortedArticles = articles.sorted { $0.name!.lowercased() < $1.name!.lowercased() }
      self.items = sortedArticles
    }
  }
}
