//
//  NewArticleViewModel.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 23/4/21.
//

import UIKit

class NewArticleViewModel {
  private var repository = ArticlesRepository.shared
  private var articles: [Article] = []
  private(set) var currentArticle: Article?

  init() {
    commonInit()
  }

  init(withRepository: ArticlesRepository) {
    repository = withRepository
    commonInit()
  }

  init(withArticle: Article) {
    commonInit()
    currentArticle = withArticle
  }

  private func commonInit() {
    repository.fetch { articles in
      self.articles = articles
    }
  }

  func saveNewArticle(withName name: String, andColor color: UIColor, completion: @escaping (StorageError) -> Void) {
    guard !name.isEmpty || name == NSLocalizedString("Give a name to your article", comment: "") else {
      return completion(.noname)
    }

    if let article = currentArticle { // update article
      guard articles.first(where: { $0.name == name && $0.id != article.id }) == nil else { return completion(.name) }

      article.name = name
      article.color = color
      repository.updateArticle { isStored in
        completion(isStored ? .none : .coreData)
      }
    } else { // create new article
      guard articles.first(where: { $0.name == name }) == nil else { return completion(.name) }

      currentArticle = repository.createNewArticle()
      currentArticle?.name = name
      currentArticle?.color = color
      repository.updateArticle { isStored in
        completion(isStored ? .none : .coreData)
      }
    }
  }

  func deleteArticle() {
    guard let article = currentArticle else { return }
    repository.delete(article: article)
  }
}
