//
//  AutomaticListGenerator.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 8/5/21.
//

import Foundation

class AutomaticListGenerator {
  static let shared = AutomaticListGenerator()

  private let repository = ShoppingListRepository.shared
  private let calendarRepository = CalendarRepository.shared
  private var shoppingLists: [ShoppingList] = []

  deinit {
    NotificationCenter.default.removeObserver(
      self,
      name: Notification.Name.NSManagedObjectContextObjectsDidChange,
      object: nil)
  }

  private init() {
    NotificationCenter.default.addObserver(
      self,
      selector: #selector(modelDidChange),
      name: Notification.Name.NSManagedObjectContextObjectsDidChange,
      object: nil)

    repository.fetchAutomaticLists { [weak self] lists in
      DispatchQueue.main.async {
        self?.shoppingLists = lists
      }
    }
  }

  func didChangeContent(inDate date: Date) {
    print("Day changed")
    print(shoppingLists)
    // print(shoppingLists.first!.dates)
    if let list = shoppingLists.first(where: {
      // swiftlint:disable:next force_unwrapping
      $0.dates!.contains { day in
        print(day, date)
        return Calendar.current.isDate(day, inSameDayAs: date)
      }
    }) {
      // list already exist for this day
      processList(list: list)
    } else {
      let newList = repository.createAutomaticList(forDate: date)
      processList(list: newList)
    }
  }

  private func processList(list: ShoppingList) {
    // get all days of week
    // swiftlint:disable:next force_unwrapping
    calendarRepository.fetch(in: list.dates!) { [weak self] days in
      // swiftlint:disable:next force_unwrapping
      guard days != nil, !days!.isEmpty else { return }

      let articlesAlreadyIncluded = list.itemsArray.compactMap { $0.article }

      var articlesOfTheWeek = Set<Article>()

      // swiftlint:disable:next force_unwrapping
      days!.forEach { day in
        day.itemsArray.forEach { item in
          if let dish = item.dish {
            dish.articlesArray.forEach { articlesOfTheWeek.insert($0) }
          }

          if let article = item.article {
            articlesOfTheWeek.insert(article)
          }
        }
      }

      guard !articlesOfTheWeek.isEmpty  else { return }

      for art in articlesOfTheWeek where !articlesAlreadyIncluded.contains(art) {
        if let newItem = self?.repository.createItem() {
          newItem.article = art
          list.addToItems(newItem)
        }
      }

      self?.repository.save { _ in }
    }
  }

  @objc private func modelDidChange() {
    repository.fetchAutomaticLists { [weak self] lists in
      self?.shoppingLists = lists
    }
  }
}
