//
//  NewDishViewModel.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 1/5/21.
//

import UIKit

class NewDishViewModel {
  private var repository = DishesRepository.shared
  private var dishes: [Dish] = []
  private(set) var currentDish: Dish? {
    didSet {
      dishDidUpdate?()
    }
  }

  var dishDidUpdate: (() -> Void)?

  private(set) var isNew = false

  init() {
    isNew = true
    commonInit()
    currentDish = repository.createDish()
  }

  init(withRepository: DishesRepository) {
    repository = withRepository
    isNew = true
    commonInit()
    currentDish = repository.createDish()
  }

  init(withDish: Dish) {
    isNew = false
    commonInit()
    currentDish = withDish
  }

  private func commonInit() {
    repository.fetch { dishes in
      self.dishes = dishes
    }
  }

  func saveNewDish(
    withName name: String,
    comment: String,
    andColor color: UIColor,
    completion: @escaping (StorageError) -> Void
  ) {
    guard !name.isEmpty else { return completion(.noname) }

    if let dish = currentDish { // update dish
      guard dishes.first(where: { $0.name == name && $0.id != dish.id }) == nil else { return completion(.name) }

      dish.name = name
      dish.color = color
      dish.comment = comment
      repository.updateDishes { isStored in
        completion(isStored ? .none : .coreData)
      }
    } else {
      guard dishes.first(where: { $0.name == name }) == nil else { return completion(.name) }

      repository.save(withName: name, color: color, andArticles: []) { isStored in
        completion(isStored ? .none : .coreData)
      }
    }
  }

  func delete(article: Article) {
    guard let dish = currentDish else { return }
    dish.removeFromArticles(article)
  }

  func discardDish() {
    guard let dish = currentDish else { return }
    repository.discard(dish: dish)
  }
}
