//
//  ListsViewModel.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 7/4/21.
//

import UIKit

class ListsViewModel: SimpleListViewModelProtocol {
  private var repository = ShoppingListRepository.shared

  var groupHeight: CGFloat = 90
  var navigationItemTitle: String = NSLocalizedString("Lists", comment: "")
  var type: ListType = .lists
  private(set) var items: [AnyHashable] = [] {
    didSet {
      itemsDidChange?(items)
    }
  }

  enum ShoppingListType: Int32 {
    case automatic = 0
    case manual = 1
  }

  var itemsDidChange: (([AnyHashable]) -> Void)?

  deinit {
    NotificationCenter.default.removeObserver(
      self,
      name: Notification.Name.NSManagedObjectContextObjectsDidChange,
      object: nil)
  }

  init(withRepository: ShoppingListRepository) {
    repository = withRepository
    commonInit()
  }

  init() {
    commonInit()
  }

  private func commonInit() {
    NotificationCenter.default.addObserver(
      self,
      selector: #selector(modelDidChange),
      name: Notification.Name.NSManagedObjectContextObjectsDidChange,
      object: nil)
  }

  func fetch() {
    repository.fetch { items in
      self.items = items
    }
  }

  func delete<T>(_ item: T) {
    if let list = item as? ShoppingList {
      repository.delete(list: list)
    }
  }

  @objc private func modelDidChange() {
    repository.fetch { lists in
      // swiftlint:disable:next force_unwrapping
      let filteredList = lists.filter { $0.name != nil && !$0.name!.isEmpty }
      self.items = filteredList.sorted {
        guard $0.dates != nil && $1.dates != nil else { return false }
        // swiftlint:disable:next force_unwrapping
        return $0.dates!.first! < $1.dates!.first!
      }
    }
  }
}
