//
//  NewShoppingListViewModel.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 4/5/21.
//

import Foundation

class NewShoppingListViewModel {
  private var listsRepository = ShoppingListRepository.shared

  private var shoppingLists: [ShoppingList] = []

  private(set) var currentList: ShoppingList {
    didSet {
      listDidUpdate?()
    }
  }

  var listDidUpdate: (() -> Void)?

  private(set) var isNew = false

  init(withRepository: ShoppingListRepository) {
    listsRepository = withRepository
    isNew = true
    currentList = listsRepository.createManualList()
    commonInit()
  }

  init() {
    isNew = true
    currentList = listsRepository.createManualList()
    commonInit()
  }

  init(withList: ShoppingList) {
    isNew = false
    currentList = withList
    commonInit()
  }

  private func commonInit() {
    listsRepository.fetch { lists in
      self.shoppingLists = lists
    }
  }

  func save(withName name: String, completion: @escaping (StorageError) -> Void) {
    guard !name.isEmpty else { return completion(.noname) }
    currentList.name = name
    listsRepository.save { isStored in
      completion(isStored ? .none : .coreData)
    }
  }

  func delete(item: ShoppingListItem) {
    item.didDiscardItem = true
  }

  func discardList() {
    listsRepository.discard(list: currentList)
  }
}
