//
//  ArticleCell.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 10/4/21.
//

import UIKit

class ArticleCell: UICollectionViewCell {
  private lazy var itemName: UILabel = {
    let lbl = UILabel()
    lbl.font = .newYorkFont(weight: .medium, andTextStyle: .title2)
    lbl.translatesAutoresizingMaskIntoConstraints = false
    lbl.backgroundColor = .clear
    lbl.textColor = .systemBackground
    return lbl
  }()

  private lazy var checkImage: UIImageView = {
    let imageView = UIImageView()
    // swiftlint:disable:next force_unwrapping
    imageView.image = UIImage(named: "checkmark")!.withRenderingMode(.alwaysTemplate)
    imageView.translatesAutoresizingMaskIntoConstraints = false
    imageView.tintColor = .systemBackground
    imageView.contentMode = .scaleAspectFit
    imageView.isHidden = true
    return imageView
  }()

  var isChecked = false {
    didSet {
      checkImage.isHidden = !isChecked
    }
  }

  override init(frame: CGRect) {
    super.init(frame: frame)
    setupViews()
  }

  required init?(coder: NSCoder) {
    super.init(coder: coder)
    setupViews()
  }

  private func setupViews() {
    layer.cornerRadius = 15
    layer.masksToBounds = true
    addSubview(itemName)
    addSubview(checkImage)

    NSLayoutConstraint.activate([
      checkImage.centerYAnchor.constraint(equalTo: centerYAnchor),
      checkImage.trailingAnchor.constraint(equalTo: trailingAnchor, constant: -10),
      checkImage.widthAnchor.constraint(equalToConstant: 30),
      checkImage.heightAnchor.constraint(equalTo: heightAnchor, constant: -20),

      itemName.topAnchor.constraint(equalTo: topAnchor, constant: 10),
      itemName.leadingAnchor.constraint(equalTo: leadingAnchor, constant: 10),
      itemName.trailingAnchor.constraint(equalTo: checkImage.leadingAnchor, constant: -10),
      itemName.bottomAnchor.constraint(equalTo: bottomAnchor, constant: -10)
    ])
  }

  func config(with article: Article) {
    itemName.text = article.name
    backgroundColor = article.wrappedColor
  }

  func config(with dish: Dish) {
    itemName.text = dish.name
    backgroundColor = dish.wrappedColor
  }
}
