//
//  CalendarMonthDayCell.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 11/4/21.
//

import UIKit

class CalendarMonthDayCell: UICollectionViewCell {
  private lazy var numberLabel: UILabel = {
    let lbl = UILabel()
    lbl.translatesAutoresizingMaskIntoConstraints = false
    lbl.textAlignment = .center
    lbl.backgroundColor = .clear
    lbl.layer.masksToBounds = true
    lbl.adjustsFontSizeToFitWidth = true
    lbl.adjustsFontForContentSizeCategory = true
    return lbl
  }()

  private var isToday = false {
    didSet {
      numberLabel.layer.borderWidth = isToday ? 1 : 0
      numberLabel.layer.borderColor = isToday ? UIColor.appLightRed.cgColor : UIColor.clear.cgColor
    }
  }

  private var isActive = false {
    didSet {
      numberLabel.backgroundColor = isActive ? .appLightRed : .clear
      numberLabel.textColor = isActive ? .systemBackground : .label
    }
  }

  private var isDisalbed = false {
    didSet {
      numberLabel.textColor = isDisalbed ? .systemGray3 : numberLabel.textColor
    }
  }

  override init(frame: CGRect) {
    super.init(frame: frame)
    setupViews()
  }

  required init?(coder: NSCoder) {
    fatalError("Not happening!")
  }

  private func setupViews() {
    addSubview(numberLabel)

    NSLayoutConstraint.activate([
      numberLabel.centerXAnchor.constraint(equalTo: centerXAnchor),
      numberLabel.centerYAnchor.constraint(equalTo: centerYAnchor),
      numberLabel.heightAnchor.constraint(equalTo: heightAnchor, constant: -4),
      numberLabel.widthAnchor.constraint(equalTo: numberLabel.heightAnchor)
    ])
  }

  override func layoutSubviews() {
    super.layoutSubviews()
    numberLabel.layer.cornerRadius = numberLabel.frame.width / 2
  }

  func config(forItem calendar: CalendarRepresentable, isTitle: Bool) {
    if isTitle {
      numberLabel.text = "\(calendar.date.dayOfWeekAbbr)"
      numberLabel.font = .newYorkFont(weight: .regular, andTextStyle: .headline)
      isActive = false
      isToday = false
      numberLabel.textColor = .systemGray2
    } else {
      numberLabel.textColor = isTitle ? .systemGray2 : .label
      numberLabel.text = "\(calendar.date.dayNumber)"
      numberLabel.font = .newYorkFont(weight: .semibold, andTextStyle: .title2)
      isToday = Calendar.current.isDateInToday(calendar.date)
      isActive = calendar.isActive
      isDisalbed = calendar.isDisabled
    }
  }
}
