//
//  CalendarWeekDayCell.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 11/4/21.
//

import UIKit

class CalendarWeekDayCell: UICollectionViewCell {
  private lazy var dayNameLabel: UILabel = {
    let lbl = UILabel()
    lbl.font = .newYorkFont(weight: .semibold, andTextStyle: .body)
    lbl.textColor = .systemGray2
    lbl.textAlignment = .center
    lbl.translatesAutoresizingMaskIntoConstraints = false
    // lbl.backgroundColor = .red
    return lbl
  }()

  private lazy var numberLabel: UILabel = {
    let lbl = UILabel()
    lbl.translatesAutoresizingMaskIntoConstraints = false
    lbl.textAlignment = .center
    // lbl.backgroundColor = .blue
    lbl.font = .newYorkFont(weight: .bold, andTextStyle: .title2)
    return lbl
  }()

  private var isActive = false {
    didSet {
      backgroundColor = isActive ? .appLightRed : .clear
      numberLabel.textColor = isActive ? .systemBackground : .label
    }
  }

  override init(frame: CGRect) {
    super.init(frame: frame)
    setupViews()
  }

  required init?(coder: NSCoder) {
    fatalError("Nope")
  }

  private func setupViews() {
    let stackView = UIStackView(arrangedSubviews: [dayNameLabel, numberLabel])
    stackView.axis = .vertical
    stackView.translatesAutoresizingMaskIntoConstraints = false

    addSubview(stackView)
    NSLayoutConstraint.activate([
      stackView.topAnchor.constraint(equalTo: topAnchor, constant: 5),
      stackView.leadingAnchor.constraint(equalTo: leadingAnchor, constant: 5),
      stackView.trailingAnchor.constraint(equalTo: trailingAnchor, constant: -5),
      stackView.bottomAnchor.constraint(equalTo: bottomAnchor, constant: -5)
    ])

    layer.cornerRadius = 15
    layer.masksToBounds = true
    layer.borderWidth = 1
    layer.borderColor = UIColor.systemGray2.cgColor
  }

  func config(forItem calendar: CalendarRepresentable) {
    numberLabel.text = "\(calendar.date.dayNumber)"
    dayNameLabel.text = "\(calendar.date.dayOfWeekAbbr)"
    isActive = calendar.isActive
  }
}
