//
//  LandingViewController.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 2/4/21.
//

import UIKit

class LandingViewController: UITabBarController {
  var rightButtonImageView: UIImageView?

  override func viewDidLoad() {
    super.viewDidLoad()

    setCustomNavigationAttrs()

    let listsViewController = ListsViewController()
    listsViewController.viewModel = ListsViewModel()
    let listsTabItem = createTabBarItem(withName: NSLocalizedString("Lists", comment: ""), andImage: "icListsTab")
    listsViewController.tabBarItem = listsTabItem

    let calendarViewController = CalendarViewController()
    let calendarTabItem = createTabBarItem(
      withName: NSLocalizedString("Calendar", comment: ""),
      andImage: "icCalendarTab")
    calendarViewController.tabBarItem = calendarTabItem

    let dishesViewController = ListsViewController()
    dishesViewController.viewModel = DishesViewModel()
    let platesTabItem = createTabBarItem(withName: NSLocalizedString("Dishes", comment: ""), andImage: "icPlatesTab")
    dishesViewController.tabBarItem = platesTabItem

    let articlesViewController = ListsViewController()
    articlesViewController.viewModel = ArticlesViewModel()
    let articlesTabItem = createTabBarItem(
      withName: NSLocalizedString("Articles", comment: ""),
      andImage: "icArticlesTab")
    articlesViewController.tabBarItem = articlesTabItem


    viewControllers = [
      listsViewController,
      calendarViewController,
      dishesViewController,
      articlesViewController
    ]
  }

  private func setCustomNavigationAttrs() {
    let attrs = [
      NSAttributedString.Key.font: UIFont.newYorkFont(weight: .semibold, andTextStyle: .body)
    ]

    let attrsLarge = [
      NSAttributedString.Key.font: UIFont.newYorkFont(weight: .bold, andTextStyle: .largeTitle)
    ]

    UINavigationBar.appearance().titleTextAttributes = attrs
    UINavigationBar.appearance().largeTitleTextAttributes = attrsLarge

    navigationItem.setHidesBackButton(true, animated: false)
    view.backgroundColor = .white

    tabBar.tintColor = .appGreen
    tabBar.barTintColor = .systemBackground
  }

  private func createTabBarItem(withName name: String, andImage image: String) -> UITabBarItem {
    let attrsTabs = [NSAttributedString.Key.font: UIFont.newYorkFont(weight: .regular, andTextStyle: .caption1)]

    let tabItem = UITabBarItem()
    tabItem.title = name
    tabItem.setTitleTextAttributes(attrsTabs, for: .normal)
    tabItem.image = UIImage(named: image)?.withRenderingMode(.alwaysTemplate)

    return tabItem
  }
}
