//
//  ShopItemCell.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 9/5/21.
//

import UIKit

class ShopItemCell: UICollectionViewCell {
  private lazy var itemName: UILabel = {
    let lbl = UILabel()
    lbl.font = .newYorkFont(weight: .regular, andTextStyle: .title3)
    lbl.translatesAutoresizingMaskIntoConstraints = false
    lbl.backgroundColor = .clear
    lbl.textColor = .label
    return lbl
  }()

  private lazy var checkImage: UIImageView = {
    let imageView = UIImageView()
    // swiftlint:disable:next force_unwrapping
    imageView.image = UIImage(systemName: "circle")!.withRenderingMode(.alwaysTemplate)
    imageView.translatesAutoresizingMaskIntoConstraints = false
    imageView.tintColor = .systemBackground
    imageView.contentMode = .scaleAspectFit
    return imageView
  }()

  var isChecked = false {
    didSet {
      if isChecked {
        checkImage.image = UIImage(systemName: "checkmark.circle.fill")?.withRenderingMode(.alwaysTemplate)
        checkImage.tintColor = .appGreen
      } else {
        checkImage.image = UIImage(systemName: "circle")?.withRenderingMode(.alwaysTemplate)
        checkImage.tintColor = .label
      }
    }
  }

  override init(frame: CGRect) {
    super.init(frame: frame)
    setupViews()
  }

  required init?(coder: NSCoder) {
    super.init(coder: coder)
    setupViews()
  }

  private func setupViews() {
    backgroundColor = .clear

    addSubview(itemName)
    addSubview(checkImage)

    NSLayoutConstraint.activate([
      checkImage.centerYAnchor.constraint(equalTo: centerYAnchor),
      checkImage.leadingAnchor.constraint(equalTo: leadingAnchor, constant: 10),
      checkImage.widthAnchor.constraint(equalToConstant: 30),
      checkImage.heightAnchor.constraint(equalToConstant: 30),

      itemName.topAnchor.constraint(equalTo: topAnchor),
      itemName.leadingAnchor.constraint(equalTo: checkImage.trailingAnchor, constant: 16),
      itemName.trailingAnchor.constraint(equalTo: trailingAnchor, constant: -16),
      itemName.bottomAnchor.constraint(equalTo: bottomAnchor)
    ])
  }

  func config(with item: ShoppingListItem) {
    if let article = item.article {
      itemName.text = article.name
    }
    isChecked = item.isPurchased
  }
}
