//
//  ShoppingListCell.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 10/4/21.
//

import UIKit

class ShoppingListCell: UICollectionViewCell {
  private lazy var itemName: UILabel = {
    let lbl = UILabel()
    lbl.font = .newYorkFont(weight: .medium, andTextStyle: .title2)
    lbl.translatesAutoresizingMaskIntoConstraints = false
    lbl.backgroundColor = .clear
    lbl.textColor = .systemBackground
    return lbl
  }()

  private lazy var itemDate: UILabel = {
    let lbl = UILabel()
    lbl.font = .newYorkFont(weight: .regularItalic, andTextStyle: .title3)
    lbl.translatesAutoresizingMaskIntoConstraints = false
    lbl.backgroundColor = .clear
    lbl.textColor = .systemBackground
    return lbl
  }()

  private lazy var checkImage: UIImageView = {
    let imageView = UIImageView()
    // swiftlint:disable:next force_unwrapping
    imageView.image = UIImage(named: "checkmark")!.withRenderingMode(.alwaysTemplate)
    imageView.translatesAutoresizingMaskIntoConstraints = false
    imageView.tintColor = .systemBackground
    imageView.contentMode = .scaleAspectFit
    return imageView
  }()

  override init(frame: CGRect) {
    super.init(frame: frame)
    setupViews()
  }

  required init?(coder: NSCoder) {
    super.init(coder: coder)
    setupViews()
  }

  private func setupViews() {
    layer.cornerRadius = 15
    layer.masksToBounds = true

    contentView.addSubview(checkImage)
    contentView.addSubview(itemName)
    contentView.addSubview(itemDate)

    NSLayoutConstraint.activate([
      checkImage.centerYAnchor.constraint(equalTo: centerYAnchor),
      checkImage.trailingAnchor.constraint(equalTo: trailingAnchor, constant: -10),
      checkImage.widthAnchor.constraint(equalToConstant: 30),
      checkImage.heightAnchor.constraint(equalTo: heightAnchor, constant: -20),

      itemName.topAnchor.constraint(equalTo: topAnchor, constant: 10),
      itemName.leadingAnchor.constraint(equalTo: leadingAnchor, constant: 10),
      itemName.trailingAnchor.constraint(equalTo: checkImage.leadingAnchor, constant: -10),
      itemName.heightAnchor.constraint(greaterThanOrEqualToConstant: 30),

      itemDate.topAnchor.constraint(equalTo: itemName.bottomAnchor, constant: 10),
      itemDate.leadingAnchor.constraint(equalTo: itemName.leadingAnchor),
      itemDate.trailingAnchor.constraint(equalTo: itemName.trailingAnchor),
      itemDate.bottomAnchor.constraint(lessThanOrEqualTo: bottomAnchor, constant: -10)
    ])

    itemName.setContentHuggingPriority(UILayoutPriority.defaultHigh, for: .vertical)
    itemDate.setContentHuggingPriority(UILayoutPriority.defaultLow, for: .vertical)
  }

  func config(with list: ShoppingList) {
    itemName.text = list.name
    itemDate.text = getDateString(forDates: list.dates)
    contentView.backgroundColor =
      list.listType == ListsViewModel.ShoppingListType.automatic.rawValue ? .appGreen : .appRed
    checkImage.isHidden = !list.isPurchased
  }

  private func getDateString(forDates dates: [Date]?) -> String {
    // swiftlint:disable force_unwrapping
    guard dates != nil && !dates!.isEmpty else { return "" }

    if dates!.count == 1 {
      return formatDate(dates!.first!)
    } else {
      let sorted = dates!.sorted()
      let firstDate = sorted.first!
      let lastDate = sorted.last!

      return "\(formatDate(firstDate)) - \(formatDate(lastDate))"
    }
    // swiftlint:enable force_unwrapping
  }

  private func formatDate(_ date: Date) -> String {
    let outputFormat = DateFormatter()
    outputFormat.timeStyle = .none
    outputFormat.dateStyle = .short
    outputFormat.locale = Locale.current
    outputFormat.doesRelativeDateFormatting = true
    return outputFormat.string(from: date)
  }

  func config(with dish: Dish) {
    itemName.text = dish.name
    // swiftlint:disable:next force_unwrapping
    let articles = dish.articlesArray.sorted { $0.name!.lowercased() < $1.name!.lowercased() }.prefix(3)

    var str = ""

    articles.forEach {
      // swiftlint:disable:next force_unwrapping
      str.append("- \($0.name!)\n")
    }

    itemDate.text = str

    itemDate.numberOfLines = 0
    contentView.backgroundColor = dish.wrappedColor
    checkImage.isHidden = true
    itemDate.sizeToFit()
  }
}
