//
//  OnboardingContentViewController.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 3/4/21.
//

import UIKit

class OnboardingContentViewController: UIViewController {
  var contentItem: OnboardingViewModel.OnboardingItem?

  private lazy var itemImageView: UIImageView = {
    let iv = UIImageView()
    iv.translatesAutoresizingMaskIntoConstraints = false
    iv.contentMode = .scaleAspectFit
    iv.tintColor = .tertiarySystemBackground
    return iv
  }()

  private lazy var itemTitle: UILabel = {
    let lbl = UILabel()
    lbl.textAlignment = .center
    lbl.textColor = .tertiarySystemBackground
    lbl.translatesAutoresizingMaskIntoConstraints = false
    lbl.font = .newYorkFont(weight: .medium, andTextStyle: .title2)
    return lbl
  }()

  private let itemMessage: UILabel = {
    let lbl = UILabel()
    lbl.textAlignment = .center
    lbl.textColor = .tertiarySystemBackground
    lbl.numberOfLines = 0
    lbl.translatesAutoresizingMaskIntoConstraints = false
    lbl.font = .newYorkFont(weight: .regular, andTextStyle: .title3)
    return lbl
  }()

  override func viewDidLoad() {
    super.viewDidLoad()

    guard contentItem != nil else {
      fatalError("OnboardingContentViewController argument requirements not filled")
    }


    setupViews()
    setupItem()
  }

  private func setupViews() {
    view.addSubview(itemImageView)
    view.addSubview(itemTitle)
    view.addSubview(itemMessage)

    NSLayoutConstraint.activate([
      itemImageView.centerXAnchor.constraint(equalTo: view.centerXAnchor),
      itemImageView.centerYAnchor.constraint(equalTo: view.centerYAnchor, constant: -150),
      itemImageView.widthAnchor.constraint(equalTo: view.widthAnchor, multiplier: 0.4),
      itemImageView.heightAnchor.constraint(equalTo: itemImageView.widthAnchor),

      itemTitle.centerXAnchor.constraint(equalTo: view.centerXAnchor),
      itemTitle.centerYAnchor.constraint(equalTo: view.centerYAnchor, constant: 50),
      itemTitle.widthAnchor.constraint(equalTo: view.widthAnchor, multiplier: 0.7),

      itemMessage.topAnchor.constraint(equalTo: itemTitle.bottomAnchor, constant: 50),
      itemMessage.centerXAnchor.constraint(equalTo: itemTitle.centerXAnchor),
      itemMessage.widthAnchor.constraint(equalTo: itemTitle.widthAnchor)
    ])
  }

  private func setupItem() {
    // swiftlint:disable force_unwrapping
    itemImageView.image = contentItem!.image
    itemTitle.text = contentItem!.title
    itemMessage.text = contentItem!.message
    view.backgroundColor = contentItem!.color
    // swiftlint:enable force_unwrapping
  }
}
