//
//  TitleAndAddHeaderView.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 9/4/21.
//

import UIKit

class TitleAndAddHeaderView: UICollectionReusableView {
  private lazy var titleLabel: UILabel = {
    let lbl = UILabel()
    lbl.font = .newYorkFont(weight: .semibold, andTextStyle: .title2)
    lbl.translatesAutoresizingMaskIntoConstraints = false
    return lbl
  }()

  private let addImage: UIImageView = {
    let iv = UIImageView()
    iv.image = UIImage(systemName: "plus")?.withRenderingMode(.alwaysTemplate)
    iv.tintColor = .label
    iv.translatesAutoresizingMaskIntoConstraints = false
    iv.contentMode = .scaleAspectFit
    iv.isUserInteractionEnabled = true
    return iv
  }()

  var addAction: (() -> Void)?

  override init(frame: CGRect) {
    super.init(frame: frame)
    setupViews()
  }

  required init?(coder: NSCoder) {
    super.init(coder: coder)
    setupViews()
  }

  private func setupViews() {
    self.backgroundColor = .systemBackground
    addSubview(titleLabel)
    addSubview(addImage)

    NSLayoutConstraint.activate([
      titleLabel.leadingAnchor.constraint(equalTo: leadingAnchor, constant: 15),
      titleLabel.topAnchor.constraint(equalTo: topAnchor, constant: 10),
      bottomAnchor.constraint(equalTo: titleLabel.bottomAnchor, constant: 5),

      addImage.topAnchor.constraint(equalTo: topAnchor, constant: 8),
      addImage.trailingAnchor.constraint(equalTo: trailingAnchor, constant: -10),
      addImage.bottomAnchor.constraint(equalTo: bottomAnchor, constant: -8),
      addImage.widthAnchor.constraint(equalToConstant: 30),

      titleLabel.trailingAnchor.constraint(equalTo: addImage.leadingAnchor, constant: -15)
    ])

    addImage.addGestureRecognizer(UITapGestureRecognizer(target: self, action: #selector(addActionHandler)))
  }

  @objc private func addActionHandler() {
    addAction?()
  }

  func configure(with title: String) {
    titleLabel.text = title
  }
}
