//
//  DishesViewModelTests.swift
//  FoodlistingTests
//
//  Created by Carlos Martinez on 12/5/21.
//

import XCTest
@testable import Foodlisting

class NewDishViewModelTests: XCTestCase {
  // swiftlint:disable implicitly_unwrapped_optional
  var coreDataStack: TestCoreDataStack!
  var sut: NewDishViewModel!
  var repository: DishesRepository!
  // swiftlint:enable implicitly_unwrapped_optional

  override func setUp() {
    super.setUp()
    coreDataStack = TestCoreDataStack()
    repository = DishesRepository(withContext: coreDataStack.persistentContainer.viewContext)
    sut = NewDishViewModel(withRepository: repository)
  }

  override func tearDown() {
    sut = nil
    super.tearDown()
  }

  func testSaveNewDish() {
    sut.saveNewDish(withName: "", comment: "", andColor: .appGreen) { error in
      XCTAssertEqual(error, .noname)
    }

    sut.saveNewDish(withName: "Test", comment: "", andColor: .appGreen) { error in
      XCTAssertEqual(error, .none, "Error guardando el plato")
      XCTAssertEqual(self.sut.currentDish?.name, "Test", "Nombre no asignado")
    }
  }

  func testDeleteArticle() {
    // swiftlint:disable force_unwrapping
    let newArticle = Article(context: coreDataStack.persistentContainer.viewContext)

    XCTAssertNotNil(sut.currentDish, "El plato no se ha creado")

    sut.currentDish!.addToArticles(newArticle)

    XCTAssertFalse(sut.currentDish!.articlesArray.isEmpty, "Artículo no añadido")

    sut.delete(article: newArticle)

    XCTAssertTrue(sut.currentDish!.articlesArray.isEmpty, "Artículo no añadido")
    // swiftlint:enable force_unwrapping
  }
}
