//
//  NewShoppingListViewModelTest.swift
//  FoodlistingTests
//
//  Created by Carlos Martinez on 12/5/21.
//

import XCTest
@testable import Foodlisting

class NewShoppingListViewModelTest: XCTestCase {
  // swiftlint:disable implicitly_unwrapped_optional
  var coreDataStack: TestCoreDataStack!
  var sut: NewShoppingListViewModel!
  // swiftlint:enable implicitly_unwrapped_optional

  override func setUp() {
    coreDataStack = TestCoreDataStack()
    let repository = ShoppingListRepository(withContext: coreDataStack.persistentContainer.viewContext)
    sut = NewShoppingListViewModel(withRepository: repository)
    super.setUp()
  }

  override func tearDown() {
    coreDataStack = nil
    sut = nil
    super.tearDown()
  }

  func testManualList() {
    XCTAssertEqual(sut.currentList.listType, ListsViewModel.ShoppingListType.manual.rawValue, "Tipo incorrecto")
    XCTAssertFalse(sut.currentList.isPurchased, "La lista se crea comprada")
    // swiftlint:disable:next force_unwrapping
    XCTAssert(Calendar.current.isDateInToday(sut.currentList.dates!.first!), "Fecha incorrecta")
  }

  func testSaveList() {
    sut.save(withName: "") { error in
      XCTAssertEqual(error, .noname, "Error comprobando el nombre")
    }

    sut.save(withName: "TestingName") { error in
      XCTAssertEqual(error, .none, "Error guardando la lista")
      XCTAssertEqual(self.sut.currentList.name, "TestingName", "Error asignando el nombre")
    }
  }

  func testDeleteList() {
    let item = ShoppingListItem(context: coreDataStack.persistentContainer.viewContext)
    XCTAssertFalse(item.didDiscardItem, "ShoppingListItem creado incorrectamente")
    sut.delete(item: item)
    XCTAssertTrue(item.didDiscardItem, "Lista no borrada")
  }
}
