//
//  Font.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 2/4/21.
//

import UIKit

// swiftlint:disable:next convenience_type
struct NewYorkFont {
  enum Weight: String {
    case black           = "NewYorkMedium-Black"
    case blackItalic     = "NewYorkMedium-BlackItalic"
    case bold            = "NewYorkMedium-Bold"
    case boldItalic      = "NewYorkMedium-BoldItalic"
    case heavy           = "NewYorkMedium-Heavy"
    case heavyItalic     = "NewYorkMedium-HeavyItalic"
    case medium          = "NewYorkMedium-Medium"
    case mediumItalic    = "NewYorkMedium-MediumItalic"
    case regular         = "NewYorkMedium-Regular"
    case regularItalic   = "NewYorkMedium-RegularItalic"
    case semibold        = "NewYorkMedium-Semibold"
    case semiboldItalic  = "NewYorkMedium-SemiboldItalic"
  }

  static let customFontSize: [UIFont.TextStyle: CGFloat] = [
    .largeTitle: 34,
    .title1: 28,
    .title2: 24,
    .title3: 20,
    .headline: 17,
    .body: 17,
    .callout: 16,
    .subheadline: 15,
    .footnote: 13,
    .caption1: 12,
    .caption2: 11
  ]
}

extension UIFont {
  class func newYorkFont(weight: NewYorkFont.Weight, andTextStyle textStyle: TextStyle) -> UIFont {
    // swiftlint:disable:next force_unwrapping
    guard let customFont = UIFont(name: weight.rawValue, size: NewYorkFont.customFontSize[textStyle]!) else {
      fatalError(
        """
        Error cargando la fuente \(textStyle.rawValue)
        """
      )
    }

    let font = UIFontMetrics(forTextStyle: textStyle).scaledFont(for: customFont)
    return font
  }
}
