//
//  UserDefault.swift
//  Foodlisting
//
//  Created by Carlos Martinez on 9/5/21.
//

import Foundation

// credit: https://www.avanderlee.com/swift/property-wrappers/
@propertyWrapper
struct UserDefault<Value> {
  let key: String
  let defaultValue: Value
  var container: UserDefaults = .standard

  var wrappedValue: Value {
    get {
      return container.object(forKey: key) as? Value ?? defaultValue
    }
    set {
      container.set(newValue, forKey: key)
    }
  }
}

extension UserDefaults {
  @UserDefault(key: "didSeeIntro", defaultValue: false)
  static var didSeeIntro: Bool

  @UserDefault(key: "calendarMode", defaultValue: CalendarViewModel.Mode.week.rawValue)
  static var calendarMode: Int
}
